/*
 Name:		HamPostaja_UHF_R1.ino
 Created:	7/17/2015 5:13:05 PM
 Author:	Marko - S56FGS
*/


#include <LCD5110_Graph/LCD5110_Graph.h>
#include <SPI/SPI.h>
//RPT names in EEPROM - Added 07.12.2015
#include <Arduino.h>
#include <EEPROM.h>
#include <DB\DB.h>

//Serial port to VHF DRA module
#define Vhf_Tx		11
#define Vhf_Rx		9
#define Vhf_RecAct	10
#define Vhf_GoTx	12

//Rotary encoder connection 
#define ENC_A		2			
#define ENC_B		3			
#define ENC_Stk		4

//Debounce time in ms
#define DebTime		25


//Nokia 5110 LCD connection
#define LCD_SCK	  A5		//A0
#define LCD_MOSI  A4		//A1
#define LCD_DC    A3		//A2
#define LCD_RST	  A2		//A3
#define LCD_CS	  A1		//A4

//RPT for VHF and UHF are stored in database.
//First record is used for storing band mode and default memory at startup
//
//EEPROM RPT memory configuration and simplex channels
//EEPROM RPT memmory MUST be split in two blocks
//First block contains VHF Data (record 2 - 25)
//Second block contains UHF Data (record 26 - 48)
//Record number equals to memory location
//Simplex memory channels follows EEPROM RPT channels
//First UHF ( memory 49 - 65) then VHF ( memory 66 - 97) 
//Frequencies are calculated from memory number.
// 

/*
#define Max_Eeprom_Memory_Nr 48

#define UHF_First_Rpt_Ch 25
#define UHF_Last_Rpt_Ch 48
#define UHF_First_Simplex_Ch 49
#define UHF_Last_Simplex_Ch 65

#define VHF_First_Rpt_Ch 2
#define VHF_Last_Rpt_Ch 24
#define VHF_First_Simplex_Ch 66
#define VHF_Last_Simplex_Ch 97
*/

#define Scan_RPT_Mems 5					//Number of scan memories minumum = 1, maximum = 12

//Nokia LCD initialisation
LCD5110 MojGLCD(LCD_SCK, LCD_MOSI, LCD_DC, LCD_RST, LCD_CS);
extern uint8_t TinyFont[];
extern uint8_t SmallFont[];
//extern uint8_t MediumNumbers[];

/*
#if (Target_Dra == 1)
	const  long int f_Rx[] = { 4385500, 4386000, 4386250, 4386750, 4387000, 4387250, 4387750, 4388000, 4388250, 4388750, 4388750, 4389000, 4389250, 4389500, 4390000, 4391250, 4391750, 4392000, 4392250, 4392250, 4392750, 4393000, 4393250 };
	const  int ctcss_tx[] = { 18, 0, 18, 18, 18, 18, 18, 18, 18, 18, 0, 18, 18, 18, 18, 18, 18, 18, 0, 0, 18, 18, 0 };
	const  char description[][14] = { "Zagarski vrh", "Mirna gora", "Stare Slemene", "Krvavec", "Grmada", "Brezice", "Krim", "RK Piran", "Precna reber", "Jance", "Korada", "Sv. Rok", "Krvavec PD", "Celje", "Laze", "Kum", "Sveta planina", "Krsko", "Pohorje", "Vojsko", "Mrzlica", "Malic", "Struska" };
	const  int name[] = { 684, 688, 690, 694, 696, 698, 702, 704, 706, 710, 710, 712, 714, 716, 720, 730, 734, 736, 738, 738, 742, 744, 746 };
	const  long int Shift_RPT = 76000;
#else
	const  long int f_Rx[] = { 1455875, 1456125, 1456125, 1456250, 1456375, 1456375, 1456500, 1456500, 1456750, 1456750, 1456750, 1456875, 1457000, 1457250, 1457250, 1457250, 1457500, 1457500, 1457625, 1457750, 1457875, 1457875, 1457875 };
	const  int ctcss_tx[] = { 18, 0, 18, 0, 18, 4, 0, 0, 0, 0, 0, 4, 18, 0, 0, 0, 18, 0, 4, 0, 18, 0, 18 };
	const  char description[][14] = { "Malic", "Mohor", "Stucinov hrib", "Nanos", "Kobla", "Stare slemene", "Trdinov vrh", "Podbrdo", "Korada", "Urslja gora", "Kranjska gora", "Zagarski vrh", "Mrzlica", "Vojsko", "Pohorje", "Jesenice", "Grmada", "Karlovica", "Izola", "Krim", "Kanin", "Mozirje", "Krsko" };
	const  int name[] = { 47, 49, 49, 50, 51, 51, 52, 52, 54, 54, 54, 55, 56, 58, 58, 58, 60, 60, 61, 62, 63, 63, 63 };
	const  long int Shift_RPT = 6000;
#endif
*/


//RPT names in EEPROM - Added 07.12.2015
//Database instance and record declaration
DB db;

#define MY_TBL 1

	struct MyRec {
		long int f_Rx;
		uint8_t ctcss_tx;
		char LongName[13];
		uint16_t ShortName;
		uint8_t Band;
} myrec;
//------------------------------------------

// Set parameters for DRA818V
int BW = 0;									// Bandwith in KHz (0 = 12.5KHz or 1 = 25KHz)
long int FrekvMode = 4334000;			    // TX frequency in MHz (134.0000 - 174.0000) for frequency mode
double FTx = 433.4000;						// TX frequency in MHz (134.0000 - 174.0000)
double FRx = 433.4000;						// RX frequency in MHz (134.0000 - 174.0000)
int RPT_Offset = 7600;						// 7.6 MHz - Default RPT offset on UHF band
long int Shift_RPT = 76000;
String tx_ctcss    = "00";				    // CTCSS frequency (0000 - 0038); 0000 = "no CTCSS"
String rx_ctcss    = "00";		    	    // CTCSS frequency (0000 - 0038); 0000 = "no CTCSS"
int squ = 4;						        // Squelch level (0 - 8); 0 = "open"

int vol = 5;						        // Initial vloume level (0-8)
byte ModeDela = 1;					        // Mode of operation (1 - Memory mode, 2 - Frequency mode, 3 - Sken local repeters)
byte RPT_Shift = 1;							// 1 Off, 2 minus, 3 plus
byte BandMode = 3;							// 1 - VHF only, 2 - UHF only, 3 - VHF and UHF 
long int RPT_OfS = 0;


String MemOpis;
String MemName;
byte znaki = 0;
long int urica = 0;					// local timer in miliseconds
int EnkTmp = 0;
byte Spomini = 27;					// Memmory number of stored chanels
byte tmpSql = 0;					// tempory squelch value
volatile byte eDir = 0;				// global rotary encoder rotation (0 - no rotation, 1- up, 2 - down)
byte Izpisi = 0;					// What to show on main screen
byte OldIzpisi = 0;					// temporary value for izpisi - we print on main screen only when izpisi != old izpisi
byte SekundeDoSkena = 0;			// Scan resume in seconds when no activity 
byte Frq_Digit = 3;					// Frequency mode - digit to adjust
byte SkenRPT = 255;					// Memory number of stored repeater (use for scaning mode)
byte ScanRPT_Memo[] = { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12 };		//Empty array for custom repeater scaning

const char Nacin[3][4] = { "MEM", "FRQ", "SCN" };			//Screen label definitin for mode of operation
const long int Frq_Add[] = { 10000, 1000, 125 };			//Frequency mode - value for each digit when adjusting


//local procedureand functions definitions
void BeriIzSpomina_UHF();									//Read data from memory location into local variables
void Nastavi_DRA();											//Send data to DRA module - frequency, squelch, tx, rx tone filters
void Nastavi_Vol_DRA();										//Send Volume level to DRA module
void OsnovniZaslon();										//Draw main screen on LCD
void ZbrisiPodrocje(byte x1, byte y1, byte x2, byte y2);	//Clear area on LCD
void Nastavitve();											//Main setup screen
void Preberi_Enkoder_1();									//Read rotation of rotary encoder
void PrikaziIzbiroModa();									//Setup - show mode of operation
void NastaviGlasnost();										//Setup - Volume 
void NastaviSquelch();										//Setup - Squelch
void NastaviMode();											//Setup - Mode of operation
void MenjajRepetitor();										//Change repeater
void NastaviTx_ctcss();										//Setup - TX ctcss
void NastaviRx_ctcss();										//Setup - RX ctcss
void ScanRPTMEM_fill();										//converts saved string to rpt scan memmory array


void setup()
{

	/* add setup code here, setup code runs once when the processor starts */
	pinMode(ENC_A, INPUT_PULLUP);
	pinMode(ENC_B, INPUT_PULLUP);
	pinMode(ENC_Stk, INPUT_PULLUP);
	pinMode(Vhf_RecAct, INPUT_PULLUP);
	pinMode(Vhf_GoTx, INPUT_PULLUP);

	MojGLCD.InitLCD();
	MojGLCD.setFont(SmallFont);
	MojGLCD.drawRoundRect(0, 0, 83, 47);
	MojGLCD.print(F("VHF/UHF"), CENTER, 2);
	MojGLCD.update();
	delay(500);
	MojGLCD.print(F("Amateur"), CENTER, 10);
	MojGLCD.update();
	delay(500);
	MojGLCD.print(F("Radio"), CENTER, 18);
	MojGLCD.update();
	delay(500);
	MojGLCD.print(F("by"), CENTER, 26);
	MojGLCD.update();
	delay(500);
	MojGLCD.print(F("S56FGS"), CENTER, 38);
	MojGLCD.update();
	delay(500);
	MojGLCD.clrScr();
	MojGLCD.update();


	Serial.begin(9600);

	attachInterrupt(0, Preberi_Enkoder_1, FALLING);

	//RPT names in EEPROM - Added 07.12.2015
	//Database open	
	db.open(MY_TBL);
	//Read setup from first record in database
	db.read(1, DB_REC myrec);
	Spomini = myrec.ShortName;
	BandMode = myrec.Band;
	ScanRPTMEM_fill(myrec.LongName);


	//--------------------------------------
/*	MojGLCD.printNumI(sizeof(myrec), CENTER, 10);
	MojGLCD.printNumI(db.nRecs(), CENTER, 18);
	MojGLCD.printNumI(sizeof(myrec)*db.nRecs(), CENTER, 26);
	MojGLCD.update();
	delay(1000);
	MojGLCD.clrScr();
	MojGLCD.update();    */
	//--------------------------------------


	// Display of main screen
	OsnovniZaslon();

	// Setup DRA Module 
	BeriIzSpomina_UHF();			//Starting in memory mode - setup DRA
	Nastavi_Vol_DRA();				//Setup volume
	urica = millis();				//start local timer
	
}

void loop()
{

	/* add main program code here, this code starts again each time it ends */
	if (ModeDela == 3)												// Scen local RPT mode
	{
		if ((millis() - urica) > 999)								//Every 1 sec we change to the next RPT 
		{
			urica = millis();
			if (SekundeDoSkena == 0)								//Time out for no activity
			{
				MenjajRepetitor();									//Set new repetear
				ZbrisiPodrocje(2, 34, 80, 35);
			}
			else
			{														// Decrement timer
				SekundeDoSkena--;
				//SekundeDoSkena--;
			}
			ShowDelayScanBar(SekundeDoSkena);
		}
	}

	if (eDir == 1)													//Encoder rotation up
	{
		if ((ModeDela == 1) || (ModeDela == 3))						//If memory or scanrpt mode
		{
			Spomini_UP();
			BeriIzSpomina_UHF();
		}
		else if (ModeDela == 2)										//if frequency mode
		{
			//FrekvMode = FrekvMode + 125;
			FrekvMode += Frq_Add[Frq_Digit-1];
				if ((FrekvMode > 1460000) && (FrekvMode < 4300000))
				{
					FrekvMode = 4300000;
				}
				if (FrekvMode > 4397750)
				{
					FrekvMode = 4397750;
				}		
			Setup_Tx_FreqMode();

			Nastavi_DRA();
		}
		eDir = 0;
	}
	if (eDir == 2)													// Encoder rotation Down
	{
		if ((ModeDela == 1) || (ModeDela == 3))						// If memory or scanrpt mode
		{
			Spomini_Down();
			BeriIzSpomina_UHF();
		}
		else if (ModeDela == 2)										// If frequency mode
		{
			//FrekvMode = FrekvMode - 125;
			FrekvMode -= Frq_Add[Frq_Digit-1];
				if ((FrekvMode < 4300000) && (FrekvMode > 1460000))
				{
					FrekvMode = 1460000;
				}
				if (FrekvMode < 1440000)
				{
					FrekvMode = 1440000;
				}
			Setup_Tx_FreqMode();
			Nastavi_DRA();
		}
		eDir = 0;
	}
	//Reding encoder pushbutton
	if (digitalRead(ENC_Stk) == LOW)
	{
		delay(DebTime);
		if (digitalRead(ENC_Stk) == LOW)
		{
			//delay(DebTime);
			urica = millis();
			znaki = true;
			do
			{
				if ((millis() - urica) > 500)					// if button presed more than 0.5 sec then go to repeater input to listen to
				{
					if (znaki == true)
					{
						tmpSql = squ;
						squ = 0;
						FRx = FTx;
						Nastavi_DRA();
						squ = tmpSql;
						znaki = false;
					}
				}
			} while (digitalRead(ENC_Stk) == LOW);

			if ((millis() - urica) < 200)					// If short click then toogle ScanRPT / Memory mode
			{
				if (ModeDela == 2)							// Frequency mode short click
				{
					Frq_Digit++;
					if (Frq_Digit == 4)
					{
						Frq_Digit = 1;
					}
				}
				else if (ModeDela == 1)						//Memory mode short click -> goto scan mode
				{
					ModeDela = 3;
					SekundeDoSkena = 2;
				}
				else if (ModeDela == 3)						//Scan mode short click -> goto memory mode
				{
					ModeDela = 1;
					ZbrisiPodrocje(3, 13, 80, 15);
				}
			}
			else if ((millis() - urica) < 500)				// If longer click then goto SetUp
			{
				if (ModeDela == 1)
				{
					BeriIzSpomina_UHF();
				}
				else
				{
					Nastavi_DRA();
				}
				Nastavitve();								// Goto SetUp screeen
			}
			if (ModeDela == 1 || ModeDela == 3)
			{
				BeriIzSpomina_UHF();
			}
			else
			{
				//tx_ctcss = "00";
				//rx_ctcss = "00";
				//FTx = (float)FrekvMode / 10000;				// TX frequency in MHz (134.0000 - 174.0000)
				//FRx = FTx;									// RX frequency in MHz (134.0000 - 174.0000)
				Setup_Tx_FreqMode();
				Nastavi_DRA();
			}
		}
		OldIzpisi = 0;										// Refesh Main screen
		Izpisi = 3;
	}

	if (digitalRead(Vhf_RecAct) == LOW)						// If Channel busy reset scan timer and dislay RX on the main screen
	{
		Izpisi = 1;
		if (ModeDela == 3)
		{
			SekundeDoSkena = 20;
		}
	}
	else if (digitalRead(Vhf_GoTx) == LOW)					// If transmitting reset scan timer an display TX on main screen
	{
		Izpisi = 2;
		if (ModeDela == 3)
		{
			SekundeDoSkena = 20;
		}

	}
	else
	{
		Izpisi = 3;											// dislay SB on main screen
	}

	if (Izpisi != OldIzpisi)								// If status changed display changes on main screen
	{
		OldIzpisi = Izpisi;
		if (Izpisi == 1)
		{
			MojGLCD.printNumF(FRx, 4, 19, 25);
			MojGLCD.invertText(true);
			MojGLCD.print(F("RX"), CENTER, 39);
			if (ModeDela != 2)
			{
				MojGLCD.print(MemOpis, CENTER, 15);
			}
			MojGLCD.invertText(false);
		}
		if (Izpisi == 2)
		{
			MojGLCD.printNumF(FTx, 4, 19, 25);
			MojGLCD.invertText(true);
			MojGLCD.print(F("TX"), CENTER, 39);
			if (ModeDela != 2)
			{
				MojGLCD.print(MemOpis, CENTER, 15);
			}
			MojGLCD.invertText(false);
		}
		if (Izpisi == 3)
		{
			MojGLCD.printNumF(FRx, 4, 19, 25);
			MojGLCD.print(F("SB"), CENTER, 39);
			if (SekundeDoSkena > 0 && ModeDela == 3)
			{
				MojGLCD.invertText(true);
			}
			if (ModeDela != 2)
			{
				MojGLCD.print(MemOpis, CENTER, 15);
			}
			MojGLCD.invertText(false);
		}
		if (ModeDela == 2)									//Frequency mode 
		{
			MojGLCD.clrLine(25, 33, 60, 33);
			if (Frq_Digit == 1)
			{
				MojGLCD.drawLine((26 + Frq_Digit * 6), 33, (26 + Frq_Digit * 6) + 5, 33);
			}
			else
			{
				MojGLCD.drawLine((26 + (Frq_Digit + 1) * 6), 33, (26 + (Frq_Digit + 1) * 6) + 5, 33);
			}
		}
		MojGLCD.update();
	}
}

