/*
 Name:		Ham_VpisBaze.ino
 Created:	12/7/2015 6:07:10 PM
 Author:	Marko
*/


#include <Arduino.h>
#include <EEPROM.h>
#include <DB\DB.h>

DB db;

#define MY_TBL 1

struct MyRec {
	long int f_Rx;
	uint8_t ctcss_tx;
	char LongName[13];
	uint16_t ShortName;
	uint8_t Band;
} myrec;

const  long int f_Rx[] = {  1455875, 1456125, 1456125, 1456250, 1456375, 1456375, 1456500, 1456500, 1456750, 1456750,
							1456750, 1456875, 1457000, 1457250, 1457250, 1457250, 1457500, 1457500, 1457625, 1457750,
							1457875, 1457875, 1457875, 1445500, 4385500, 4386000, 4386250, 4386750, 4387000, 4387250,
							4387750, 4388000, 4388250, 4388750, 4388750, 4389000, 4389250, 4389500, 4390000, 4391250,
							4391750, 4392000, 4392250, 4392250, 4392750, 4393000, 4393250 };
const  int ctcss_tx[] = {  18, 0, 18, 0, 18, 4, 0, 0, 0, 0,
							0, 4, 18, 0, 0, 0, 18, 0, 4, 0,
							18, 0, 18, 18, 18, 0, 18, 18, 18, 18,
							18, 18, 18, 18, 0, 18, 18, 18, 18, 18,
							18, 18, 0, 0, 18, 18, 0 };
const  char description[][14] = {   "Malic", "Mohor", "Stucinov hrb", "Nanos", "Kobla", "Stare Slemen", "Trdinov vrh", "Podbrdo", "Korada", "Urslja gora",
									"Kranjska gor", "Zagarski vrh", "Mrzlica", "Vojsko", "Pohorje", "Jesenice", "Grmada", "Karlovica", "Izola", "Krim",
									"Kanin", "Mozirje", "Krsko", "Echolnk LJ", "Zagarski vrh", "Mirna gora", "Stare Slemen", "Krvavec", "Grmada", "Brezice",
									"Krim", "RK Piran", "Precna reber", "Jance", "Korada", "Sv. Rok", "Krvav PD", "Celje", "Laze", "Kum",
									"Sv. planina", "Krsko", "Pohorje", "Vojsko", "Mrzlica", "Malic", "Struska" };
const  int name[] = {   47, 49, 49, 50, 51, 51, 52, 52, 54, 54,
						54, 55, 56, 58, 58, 58, 60, 60, 61, 62,
						63, 63, 63, 99, 684, 688, 690, 694, 696, 698,
						702, 704, 706, 710, 710, 712, 714, 716, 720, 730,
						734, 736, 738, 738, 742, 744, 746 };


// the setup function runs once when you press reset or power the board
void setup() {
	// String tmpLongName;
	Serial.begin(9600);
	Serial.println("SLO - VHF/UHF repeater database");
	Serial.println();
	
	//randomSeed(analogRead(0));

	Serial.print("Creating Table...");
	db.create(MY_TBL, sizeof(myrec));
	db.open(MY_TBL);
	Serial.println("DONE");

	// First record is setup record used in setup
	Serial.print("SetUp record...");
	myrec.Band = 3;									// 3 - VHF and UHF mode
	myrec.ctcss_tx = 1;								//	Memory location on power on
	myrec.f_Rx = 1440000;							// not used
	String tmpLongName = "S56FGS";							// not used
	tmpLongName.toCharArray(myrec.LongName, 14);	// not used
	myrec.ShortName = 1;							// not used
	db.append(DB_REC myrec);
	Serial.println("DONE");

	
	Serial.println("Creating records...");
	for (int recno = 1; recno <= 47; recno++)				//1-24 --> VHF, 25-47 --> UHF
	{
		String tmpLongName = description[recno - 1];   //"Malic";
		myrec.f_Rx = f_Rx[recno - 1];
		myrec.ctcss_tx = ctcss_tx[recno - 1];
		tmpLongName.toCharArray(myrec.LongName, 14);
		myrec.ShortName = name[recno - 1];
		if (name[recno - 1] < 200)
		{
			myrec.Band = 1;					// VHF 
		}
		else
		{
			myrec.Band = 2;					// UHF 
		}
		Serial.print("Recnum: "); Serial.println(recno);
		Serial.print("Frekvenca: "); Serial.println(myrec.f_Rx);
		Serial.print("CTCSS_Tx: "); Serial.println(myrec.ctcss_tx);
		Serial.print("Ime: "); Serial.println(myrec.LongName);
		Serial.print("RV: "); Serial.println(myrec.ShortName);
		Serial.print("Band: "); Serial.println(myrec.Band);
		Serial.println("-----");
		db.append(DB_REC myrec);
		Serial.print("DONE record: ");
		Serial.println(recno + 1);
	}
	
	/*
	//RPT 1
	tmpLongName = "Malic";
	myrec.f_Rx = 1455875;
	myrec.ctcss_tx = 18;
	tmpLongName.toCharArray(myrec.LongName, 14);
	myrec.ShortName = 47;
	db.append(DB_REC myrec);
	//RPT 2
	tmpLongName = "Mohor";
	myrec.f_Rx = 1455875;
	myrec.ctcss_tx = 18;
	tmpLongName.toCharArray(myrec.LongName, 14);
	myrec.ShortName = 47;
	db.append(DB_REC myrec);

	Serial.println("DONE");

	
	for (int recno = 1; recno <= 10; recno++)
	{
	
		
		int m = random(1, 12);
		int d = random(1, 31);
		int h = random(1, 12);
		int i = random(59);
		int s = random(59);

		sprintf(myrec.date, "2009-%02d-%02d", m, d);
		sprintf(myrec.time, "%02d:%02d:%02d", h, i, s);
		myrec.temperature = random(-100, 100);
		Serial.print("CREATING RECNUM: "); Serial.println(recno);
		Serial.print("WRITING Date: "); Serial.println(myrec.date);
		Serial.print("WRITING Time: "); Serial.println(myrec.time);
		Serial.print("WRITING Temperature: "); Serial.println(myrec.temperature);
		db.append(DB_REC myrec);
		Serial.println("DONE");
	}
	*/
	Serial.println();
	Serial.println("Reading records from EEPROM...");
	Serial.println();
	//db.open(MY_TBL);
	//delay(5000);
	delay(5000);
	//delay(5000);
	selectAll();
}

// the loop function runs over and over again until power down or reset
void loop() {
	//delay(5000);
	//selectAll();
	//delay(5000);
	//delay(5000);

  
}


void selectAll()
{
	if (db.nRecs()) Serial.println("-----");
	Serial.print("Stevilo zapisov v bazi:");
	Serial.println(db.nRecs());
	Serial.println("-----");

	Serial.print("Velikost zapisa:");
	Serial.println(sizeof(myrec), DEC);
	Serial.println("-----");

	for (int i = 1; i <= db.nRecs(); i++)
	{
		db.read(i, DB_REC myrec);
		Serial.print("Recnum: "); Serial.println(i);
		Serial.print("Frekvenca: "); Serial.println(myrec.f_Rx);
		Serial.print("CTCSS_Tx: "); Serial.println(myrec.ctcss_tx);
		Serial.print("Ime: "); Serial.println(myrec.LongName);
		Serial.print("RV: "); Serial.println(myrec.ShortName);
		Serial.print("Band: "); Serial.println(myrec.Band);
		Serial.println("-----");
	}
	Serial.println("Done reading!");
}